#include <windows.h>
#include <stdio.h>
#include <fstream>
#include <wx/wx.h>
#include <wx/image.h>
#include <wx/wizard.h>
#include <wx/listbox.h>
#include "bzip.hpp"
#include "tar.hpp"
#include "DFArcFrame.hpp"
#include "dfdizwindow.hpp"
#include "InstallVerifyFrame.hpp"
#include "RecursiveDelete.hpp"
#include "Options.hpp"
#include "Package.hpp"

// FRAME EVENT TABLE
BEGIN_EVENT_TABLE(DFArcFrame, wxFrame)
    EVT_MENU(wxID_EXIT, DFArcFrame::OnQuit)
    EVT_MENU(wxID_ABOUT, DFArcFrame::OnAbout)
    EVT_MENU(DFArcFrame::Identifier::ID_FileInstall, DFArcFrame::Install)
    EVT_MENU(DFArcFrame::Identifier::ID_Download, DFArcFrame::onDownload)
    EVT_MENU(DFArcFrame::Identifier::ID_Refresh, DFArcFrame::onRefresh)
    EVT_MENU(DFArcFrame::Identifier::ID_Uninstall, DFArcFrame::uninstall)
    EVT_MENU(DFArcFrame::Identifier::ID_Options, DFArcFrame::showOptions)
    EVT_MENU(DFArcFrame::Identifier::ID_Introduction, DFArcFrame::showIntroduction)
    EVT_MENU(DFArcFrame::Identifier::ID_Walkthroughs, DFArcFrame::onWalkthroughs)
    EVT_MENU(DFArcFrame::Identifier::ID_Forums, DFArcFrame::onForums)
    EVT_MENU(DFArcFrame::Identifier::ID_Browse, DFArcFrame::onBrowse)
   
    EVT_BUTTON(DFArcFrame::Identifier::ID_Play, DFArcFrame::OnPlay)
    EVT_BUTTON(DFArcFrame::Identifier::ID_Edit, DFArcFrame::onEdit)
    EVT_BUTTON(DFArcFrame::Identifier::ID_Package, DFArcFrame::onPackage)
    
    EVT_LISTBOX(DFArcFrame::Identifier::ID_DmodTitleList, DFArcFrame::ChangeDescription)
    
    EVT_CHECKBOX(DFArcFrame::Identifier::ID_Debug, DFArcFrame::OnSetPlayOption)
    EVT_CHECKBOX(DFArcFrame::Identifier::ID_Sound, DFArcFrame::OnSetPlayOption)
    EVT_CHECKBOX(DFArcFrame::Identifier::ID_Joystick, DFArcFrame::OnSetPlayOption)
    EVT_CHECKBOX(DFArcFrame::Identifier::ID_Truecolor, DFArcFrame::OnSetPlayOption)
    EVT_CHECKBOX(DFArcFrame::Identifier::ID_Windowed, DFArcFrame::OnSetPlayOption)
    EVT_SHOW(DFArcFrame::onShow)
END_EVENT_TABLE()

///////////////////////////////////////////////////////////////////////////////////////////////////
// Constructor
//
DFArcFrame::DFArcFrame(const wxString &wxsTitle, const wxPoint &wxpPos, const wxSize &wxsSize) :
	wxFrame(0, -1, wxsTitle, wxpPos, wxsSize, wxDEFAULT_FRAME_STYLE),
	mDebugValue( false ),
	mJoystickValue( true ),
	mSoundValue( true ),
	mTrueColorValue( false ),
	mWindowedValue( false ),
	mSelectedDmod( "" ),
    mWriteIniValue( true ),
    mPlayMovieValue( false ),
    mListAllSubdirectories( false ),
    mOverrideDinkDirectory( false ),
    mSpecifiedDinkDirectory( "" ),
    mDinkExe( "dink.exe" ),
    mEditorExe( "dinkedit.exe" ),
    mNoDmods( false ),
    mShowDeveloperButtons( false ),
    mShowIntroduction( true ),
    mCloseDfarcOnPlay( false ),
    mWarnOnEdit( true )
{
    mConfig = new wxConfig( "DFArc2" );
    initializeConfig();

	DFS = new DFilesystem( &mListAllSubdirectories, &mOverrideDinkDirectory, &mSpecifiedDinkDirectory );
    chdir( DFS->getDinkPath().c_str() );

	PrepareGUI();
    selectDmod();
	updateConfig();
	return;
}

void DFArcFrame::initializeConfig( void )
{
	mConfig->Read( "Debug", &mDebugValue );
	mConfig->Read( "Joystick", &mJoystickValue );
	mConfig->Read( "Sound", &mSoundValue );
	mConfig->Read( "True Color", &mTrueColorValue );
	mConfig->Read( "Windowed", &mWindowedValue );
	mConfig->Read( "Last Selected D-Mod", &mSelectedDmod );
    mConfig->Read( "Show Introduction", &mShowIntroduction );
    mConfig->Read( "Warn on Edit", &mWarnOnEdit );

    // Options dialog
    mConfig->Read( "Write Ini", &mWriteIniValue );
    mConfig->Read( "Play Movie", &mPlayMovieValue );
    mConfig->Read( "Show Developer Buttons", &mShowDeveloperButtons );
    mConfig->Read( "List All Subdirectories", &mListAllSubdirectories );
    mConfig->Read( "Close DFArc on Play", &mCloseDfarcOnPlay );
    mConfig->Read( "Override Dink Directory", &mOverrideDinkDirectory );
    mConfig->Read( "Specified Dink Directory", &mSpecifiedDinkDirectory );
    mConfig->Read( "Dink Executable", &mDinkExe );
    mConfig->Read( "Editor Executable", &mEditorExe );
	
	return;
}

void DFArcFrame::updateDescription( unsigned int aSelectedDmodIndex )
{
	std::fstream lFileStream;
    wxString lDescription( "" );
    mSelectedDmod = "";

	// Set the current D-Mod directory to the currently selected item's client string
    if ( mFolderTitlePairs->size() > 0 )
    {
	    wxStringClientData* lDirectory = reinterpret_cast< wxStringClientData* >( mDmodList->GetClientObject( aSelectedDmodIndex ) );
	    mSelectedDmod = lDirectory->GetData();
    }

    for ( std::vector< std::pair< wxString, wxString > >::const_iterator lIterator = mFolderTitlePairs->begin();
        lIterator != mFolderTitlePairs->end();
        lIterator++ )
    {
        wxString lCurrentPath( lIterator->first );
        if ( lCurrentPath == mSelectedDmod )
        {
            lFileStream.open( DFS->getDinkPath() + lCurrentPath + "dmod.diz", std::ios::in );
            char lCharacter;
            while( ( lCharacter = lFileStream.get() ) != EOF )
            {
                lDescription += lCharacter;
            }
            lFileStream.close();
            break;
        }
    }

	mDmodDescription->SetValue( lDescription );
	updateConfig();
    mStatusBar->SetStatusText( DFS->getDinkPath() + mSelectedDmod );

    wxString lDmodPreview( mSelectedDmod + "preview.bmp" );
    wxString lDmodTitle( mSelectedDmod + "graphics/title-01.bmp" );
    if ( ::wxFileExists( lDmodPreview ) == true )
    {
        mLogoBitmap = wxBitmap( lDmodPreview, wxBITMAP_TYPE_BMP );
    }
    else if ( ::wxFileExists( lDmodTitle ) == true )
    {
        wxImage lImage( lDmodTitle );
        lImage.Rescale( 160, 120 );
        mLogoBitmap = wxBitmap( lImage );
    }
    else
    {
        mLogoBitmap = wxBitmap( 160, 120 );
    }
    //mLogoButton->SetBitmapDisabled( mLogoBitmap );
    mLogoBitmap.SetWidth( 160 );
    mLogoBitmap.SetHeight( 120 );
    mLogoButton->SetBitmapFocus( mLogoBitmap );
    mLogoButton->SetBitmapLabel( mLogoBitmap );
    mLogoButton->SetBitmapSelected( mLogoBitmap );
	return;
}

void DFArcFrame::updateConfig( void )
{
	if ( mWindowedValue == true )
	{
		mTrueColorValue = true;
		mTrueColor->Disable();
	}
	else
	{
		mTrueColor->Enable();
	}

	// Update the checkboxes
	mDebug->SetValue( mDebugValue );
	mJoystick->SetValue( mJoystickValue );
	mSound->SetValue( mSoundValue );
	mWindowed->SetValue( mWindowedValue );
	mTrueColor->SetValue( mTrueColorValue );

	// Update config
	mConfig->Write( "Debug", mDebugValue );
	mConfig->Write( "Joystick", mJoystickValue );
	mConfig->Write( "Sound", mSoundValue );
	mConfig->Write( "Windowed", mWindowedValue );
	mConfig->Write( "True Color", mTrueColorValue );
    mConfig->Write( "Show Introduction", mShowIntroduction );
    mConfig->Write( "Warn on Edit", mShowIntroduction );

    // Options dialog
    mConfig->Write( "Write Ini", mWriteIniValue );
    mConfig->Write( "Play Movie", mPlayMovieValue );
    mConfig->Write( "Show Developer Buttons", mShowDeveloperButtons );
    mConfig->Write( "List All Subdirectories", mListAllSubdirectories );
    mConfig->Write( "Close DFArc on Play", mCloseDfarcOnPlay );
    mConfig->Write( "Override Dink Directory", mOverrideDinkDirectory );
    mConfig->Write( "Specified Dink Directory", mSpecifiedDinkDirectory );
    mConfig->Write( "Dink Executable", mDinkExe );
    mConfig->Write( "Editor Executable", mEditorExe );

	mConfig->Write( "Last Selected D-Mod", mSelectedDmod );
	return;
}

// VIRTUAL OVERRIDES
void DFArcFrame::OnQuit(wxCommandEvent& aCommandEvent)
{
    this->Close(true);
}

void DFArcFrame::OnAbout(wxCommandEvent& aCommandEvent)
{
     // Use a wxString for concatenation with the date.
     wxString wxs(wxT("DFArc version 2.0\nCopyright (C) 2005 Andrew Reading\nCreated by Andrew Reading (Merlin) and Dan Walma (redink1)\nBuild Date: "));
     wxMessageBox(wxs + wxT(__DATE__) + "\nPowered by bzip2 ( http://www.bzip.org ) and wxWidgets ( http://www.wxwidgets.org )", wxT("About DFArc2"), 
         wxOK | wxICON_INFORMATION, this);
}


// GUI for main screen (with no args).
void DFArcFrame::PrepareGUI()
{   
    // Set the icon
    this->SetIcon( wxIcon( "DINK" ) );

    // First, make the menubar
    wxMenu* lFileMenu = new wxMenu();
    lFileMenu->Append( Identifier::ID_FileInstall, "&Open D-Mod to Install", "Select a D-Mod to install" );
    lFileMenu->Append( Identifier::ID_Download, "&Download D-Mods", "Go to The Dink Network to download some D-Mods!" );
    lFileMenu->AppendSeparator();
    lFileMenu->Append( wxID_EXIT, "E&xit", "Exit DFArc2" );

    wxMenu* lEditMenu = new wxMenu();
    lEditMenu->Append( Identifier::ID_Refresh, "&Refresh D-Mod List", "Refreshes the D-Mod list for any new additions" );
    lEditMenu->Append( Identifier::ID_Browse, "&Browse Selected D-Mod Directory", "Browse to the directory containing the current D-Mod" );
    lEditMenu->Append( Identifier::ID_Uninstall, "&Uninstall Selected D-Mod", "Uninstalls the selected D-Mod" );
    lEditMenu->AppendSeparator();
    lEditMenu->Append( Identifier::ID_Options, "&Options", "View or modify DFArc2 options" );

    wxMenu* lHelpMenu = new wxMenu();
    lHelpMenu->Append( Identifier::ID_Introduction, "&Introduction", "A quick introduction to Dinking and D-Mods" );
    lHelpMenu->Append( Identifier::ID_Walkthroughs, "&Walkthroughs and Guides", "Stuck in a D-Mod? Check out The Dink Smallwood Solutions." );
    lHelpMenu->Append( Identifier::ID_Forums, "&Forums", "Ask a question on the forums, or see if someone had the same problem." );
    lHelpMenu->AppendSeparator();
    lHelpMenu->Append( wxID_ABOUT, "&About", "About DFArc2" );

    wxMenuBar* lMenuBar = new wxMenuBar();
    lMenuBar->Append(lFileMenu, "&File");
    lMenuBar->Append(lEditMenu, "&Edit");
    lMenuBar->Append(lHelpMenu, "&Help");
            
    SetMenuBar( lMenuBar );

    // Make the status bar
    mStatusBar = new wxStatusBar( this, -1 );
    SetStatusBar( mStatusBar );

    // Make a new panel
    wxPanel* lPanel = new wxPanel(this);
     
	// Now the DMOD list.
    mDmodList = new wxListBox(lPanel, Identifier::ID_DmodTitleList, wxDefaultPosition, wxSize( 150, 165 ),
        0, NULL, wxLB_SINGLE | wxLB_NEEDED_SB | wxLB_SORT);
	refreshDmodList();
	
    mPlayButton = new wxButton(lPanel, Identifier::ID_Play, wxT("Play"));
    mTrueColor = new wxCheckBox(lPanel, Identifier::ID_Truecolor, wxT("True Color"));
    mWindowed = new wxCheckBox(lPanel, Identifier::ID_Windowed, wxT("Windowed"));
    mSound = new wxCheckBox(lPanel, Identifier::ID_Sound, wxT("Sound"));
    mJoystick = new wxCheckBox(lPanel, Identifier::ID_Joystick, wxT("Joystick"));
    mDebug = new wxCheckBox(lPanel, Identifier::ID_Debug, wxT("Debug"));

    // Play sizer
	wxBoxSizer* lPlaySizer = new wxBoxSizer( wxVERTICAL );
    lPlaySizer->Add( mPlayButton, wxSizerFlags( 0 ).Border( wxBOTTOM, 10 ).Left() );
    lPlaySizer->Add( mTrueColor, wxSizerFlags( 0 ).Border( wxBOTTOM, 7 ).Left() );
    lPlaySizer->Add( mWindowed, wxSizerFlags( 0 ).Border( wxBOTTOM, 7 ).Left() );
    lPlaySizer->Add( mSound, wxSizerFlags( 0 ).Border( wxBOTTOM, 7 ).Left() );
    lPlaySizer->Add( mJoystick, wxSizerFlags( 0 ).Border( wxBOTTOM, 7 ).Left() );
    lPlaySizer->Add( mDebug, wxSizerFlags( 0 ).Border( wxBOTTOM, 7 ).Left() );

    mEditButton = new wxButton( lPanel, Identifier::ID_Edit, wxT( "Edit" ) );
    mPackageButton = new wxButton( lPanel, Identifier::ID_Package, wxT( "Package" ) );

    // Button Sizer
    mButtonSizer = new wxBoxSizer( wxHORIZONTAL );
    mButtonSizer->Add( mEditButton, wxSizerFlags( 0 ).Border( wxLEFT, 10 ).Left() );
    mButtonSizer->Add( mPackageButton, wxSizerFlags( 0 ).Border( wxLEFT, 10 ).Left() );

    mLogoButton = new wxBitmapButton( lPanel, -1, mLogoBitmap, wxDefaultPosition, wxSize( 160, 120 ), wxNO_BORDER );

    // Image Sizer
    mImageSizer = new wxBoxSizer( wxVERTICAL );
    mImageSizer->Add( mButtonSizer, wxSizerFlags( 0 ).Border( wxDOWN, 10 ).Left() );
    mImageSizer->Add( mLogoButton, wxSizerFlags( 0 ).Border( wxLEFT, 10 ).Left() );
    mImageSizer->Show( mButtonSizer, mShowDeveloperButtons );
    mImageSizer->Layout();
    
    // Top Sizer
    wxBoxSizer* lTopSizer = new wxBoxSizer( wxHORIZONTAL );
	lTopSizer->SetMinSize( 400, 165 );
    lTopSizer->Add( mDmodList, wxSizerFlags( 1 ) .Expand() );
    lTopSizer->Add( lPlaySizer, wxSizerFlags( 0 ).Border( wxLEFT, 10 ).Left() );
    lTopSizer->Add( mImageSizer, wxSizerFlags( 0 ).Left() );

    // Dmod Description
    mDmodDescription = new wxTextCtrl(lPanel, -1, "", wxDefaultPosition,
        wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY | wxTE_LEFT | wxTE_BESTWRAP);
    
    // Add the sizers to a main sizer.
    DFMainSizer = new wxBoxSizer(wxVERTICAL);
	DFMainSizer->SetDimension(0, 0, 250, 300);
    DFMainSizer->Add(lTopSizer, wxSizerFlags(0).Border(wxALL, 5).Expand());
    DFMainSizer->Add(mDmodDescription, wxSizerFlags(1).Border(wxALL, 5).Expand());
    
    lPanel->SetSizer(DFMainSizer);
	lPanel->SetSize(425, 400);
	this->SetSize(425, 400);
	this->SetMinSize( wxSize( 425, 400 ) );
}


void DFArcFrame::ChangeDescription(wxCommandEvent &Event)
{
	updateDescription( Event.GetSelection() );
	return;
}

// Function for Install button.
void DFArcFrame::Install( wxCommandEvent& aCommandEvent )
{
    wxFileDialog FileDlg(0, "Select a .dmod file", "", "", "D-Mod files (*.dmod)|*.dmod",
        wxOPEN | wxFILE_MUST_EXIST | wxHIDE_READONLY);
     
    if ( FileDlg.ShowModal() == wxID_OK )
    {
        InstallVerifyFrame* lTemp = new InstallVerifyFrame( this, FileDlg.GetPath(), DFS );
        lTemp->ShowModal();
        wxString lSelectedDmod( lTemp->getSelectedDmod() );
        lTemp->Destroy();

        if ( lSelectedDmod.Len() > 0 )
        {
            mSelectedDmod = lSelectedDmod;
        }
        
	    //InstallFrame* lInstallFrame = new InstallFrame( FileDlg.GetPath(), DFS );
        //mSelectedDmod = lInstallFrame->getInstalledDmodDirectory();
        //lInstallFrame->Destroy();

		// Update the D-Mod list
		refreshDmodList();
        selectDmod();
   }
}

void DFArcFrame::onPackage(wxCommandEvent &Event)
{
    Package lPackage( mSelectedDmod.Mid( 0, mSelectedDmod.Len() - 1 ), mDmodDescription->GetValue(), DFS );
    lPackage.ShowModal();
    /*

    if(PkgDirDropdown->GetValue() == "" || PkgNameText->GetValue() == "")
    {
        ::wxLogMessage("You left some of the fields blank.");
        return;
    }
    
    if(!DFS->IsValidFilename(PkgNameText->GetValue()))
    {
        ::wxLogMessage("'%s' is an invalid filename.  Please make corrections.",
            (PkgNameText->GetValue() + ".dmod").c_str());
        return;
    }


    // TEMPORARILY DISABLE THE PACKAGE BUTTON
    PkgButton->Disable();

    
    PkgStatusBar->SetValue(0);
    PkgStatusLabel->SetLabel("Creating archive...");

    ::wxYield();

    
    // Tar creation code.
    Tar *tfile = new Tar(0, DFS, (DFS->getDinkPath() + PkgDirDropdown->GetValue()).c_str());
    wxString strTarFile = tfile->Create( NULL );
    delete tfile;
    
    PkgStatusBar->SetValue(0);
    PkgStatusLabel->SetLabel("Compressing...");

    ::wxYield();
        
    // Prepare for an absolute path.
    wxString strCwd = ::wxGetCwd();
    DFS->FixSlashes(strCwd);
    
    // Bzip stuff
    Bzip *Bz = new Bzip(strTarFile.c_str(), DFS);
    wxString strBzFile = Bz->Create( NULL );  
    
    // Make a new object for the copy ctor.
    Bz->MoveFile(strBzFile.c_str(), (strCwd + PkgNameText->GetValue() + ".dmod").c_str());
    delete Bz;
    
    PkgStatusBar->SetValue(0);
    PkgStatusLabel->SetLabel("Success!");


    PkgButton->Enable();*/
}

void DFArcFrame::OnSetPlayOption(wxCommandEvent &Event)
{
    // Set variables
	mDebugValue = mDebug->IsChecked();
	mJoystickValue = mJoystick->IsChecked();
	mSoundValue = mSound->IsChecked();
	mWindowedValue = mWindowed->IsChecked();
	mTrueColorValue = mTrueColor->IsChecked();
	
	updateConfig();
	return;
}


void DFArcFrame::OnPlay(wxCommandEvent &Event)
{
    if ( ::wxFileExists( mDinkExe ) == false )
    {
        ::wxLogError( "The .exe file you selected to launch Dink Smallwood could not be found. Please specify a valid executable in the Options menu." );
        return;
    }

    wxString lCommand( mDinkExe );

    if ( mDebugValue == true )
    {
        lCommand += " -debug";
    }
    if ( mSoundValue == false )
    {
        lCommand += " -nosound";
    }
	if ( mTrueColorValue == true )
    {
        lCommand += " -truecolor";
    }
	if ( mWindowedValue == true )
    {
        lCommand += " -window";
    }
    if ( mPlayMovieValue == false )
    {
        lCommand += " -nomovie";
    }
    if ( mWriteIniValue == false )
    {
        lCommand += " -noini";
    }
    if ( mJoystickValue == false )
    {
        lCommand += " -nojoy";
    }

    lCommand += " -game ";
    lCommand += mSelectedDmod;
       
    // Remove the slash on the end
    lCommand.Truncate( lCommand.size() - 1 );

    STARTUPINFO lStartupInfo;
    ZeroMemory( &lStartupInfo, sizeof( lStartupInfo ) );
    lStartupInfo.cb = sizeof( lStartupInfo );
    PROCESS_INFORMATION lProcessInformation;
    ZeroMemory( &lProcessInformation, sizeof(lProcessInformation) );

    // Start the child process. 
    if( !CreateProcess( NULL,   // No module name (use command line). 
        LPSTR( lCommand.c_str() ),  // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        LPSTR( DFS->getDinkPath().c_str() ),             // Use parent's starting directory. 
        &lStartupInfo,              // Pointer to STARTUPINFO structure.
        &lProcessInformation )             // Pointer to PROCESS_INFORMATION structure.
    ) 
    {
        ::wxLogError( "Could not start dink: %s.\n", GetLastError() );
    }
    else
    {
        if ( mCloseDfarcOnPlay == true )
        {
            this->Close(true);
        }
    }
}

void DFArcFrame::onEdit( wxCommandEvent& aEvent )
{
    if ( ::wxFileExists( mEditorExe ) == false )
    {
        ::wxLogError( "The editor executable could not be found. Please specify a valid executable in the Options menu." );
        return;
    }

    bool lEdit( true );
    if ( mWarnOnEdit == true )
    {
        if ( wxMessageBox( "Dinkedit saves all changes automatically. Altering maps can ruin the game. Are you sure you want to continue?", "Warning", 
            wxOK | wxCANCEL | wxICON_INFORMATION, this ) == wxCANCEL )
        {
            lEdit = false;
        }
        else
        {
            mWarnOnEdit = false;
            updateConfig();
        }
    }
    if ( lEdit == true )
    {
        wxString lCommand( mEditorExe );
        lCommand += " -game ";
        lCommand += mSelectedDmod;

        // Remove the slash on the end
        lCommand.Truncate( lCommand.size() - 1 );

        STARTUPINFO lStartupInfo;
        ZeroMemory( &lStartupInfo, sizeof( lStartupInfo ) );
        lStartupInfo.cb = sizeof( lStartupInfo );
        PROCESS_INFORMATION lProcessInformation;
        ZeroMemory( &lProcessInformation, sizeof(lProcessInformation) );

        // Start the child process. 
        if( !CreateProcess( NULL,   // No module name (use command line). 
            LPSTR( lCommand.c_str() ),  // Command line. 
            NULL,             // Process handle not inheritable. 
            NULL,             // Thread handle not inheritable. 
            FALSE,            // Set handle inheritance to FALSE. 
            0,                // No creation flags. 
            NULL,             // Use parent's environment block. 
            LPSTR( DFS->getDinkPath().c_str() ),             // Use parent's starting directory. 
            &lStartupInfo,              // Pointer to STARTUPINFO structure.
            &lProcessInformation )             // Pointer to PROCESS_INFORMATION structure.
        ) 
        {
            ::wxLogError( "Could not start editor.\n" );
        }
    }
}

void DFArcFrame::onRefresh( wxCommandEvent& aEvent )
{
    refreshDmodList();
    selectDmod();
    return;
}

void DFArcFrame::onDownload( wxCommandEvent& aEvent )
{
    ::ShellExecute(0, _T("open"), "http://www.dinknetwork.com/?src=DFArc2", NULL, NULL, SW_SHOW);
    return;
}

void DFArcFrame::onWalkthroughs( wxCommandEvent& aEvent )
{
    ::ShellExecute(0, _T("open"), "http://solutions.dinknetwork.com/?src=DFArc2", NULL, NULL, SW_SHOW);
    return;
}

void DFArcFrame::onForums( wxCommandEvent& aEvent )
{
    ::ShellExecute(0, _T("open"), "http://www.dinknetwork.com/forum.cgi?src=DFArc2", NULL, NULL, SW_SHOW);
    return;
}

void DFArcFrame::onBrowse( wxCommandEvent& aEvent )
{
    wxString lCurrentDirectory = DFS->getDinkPath();
    if ( mSelectedDmod != "dink\\" )
    {
        lCurrentDirectory += mSelectedDmod;
    }
    ::ShellExecute(0, _T("open"), lCurrentDirectory.c_str(), NULL, NULL, SW_SHOW);
    return;
}

void DFArcFrame::showIntroduction( wxCommandEvent& aEvent )
{
    wxMessageBox("Welcome to DFArc2, the Dink Smallwood front end!\n\nYou can choose to play the original game, Dink Smallwood, or Dink-Modules (D-Mods) which contain new adventures.  After completing the main game, give the included D-Mod (Mystery Island) a try.\n\nThere are hundreds of free D-Mods available, just click File-Download D-Mods.", "Introduction", 
         wxOK | wxICON_INFORMATION, this);
    return;
}

void DFArcFrame::onShow( wxShowEvent& aEvent )
{
    if ( mShowIntroduction == true )
    {
        showIntroduction( wxCommandEvent() );
        mShowIntroduction = false;
        updateConfig();
    }
    return;
}

void DFArcFrame::refreshDmodList( void )
{
	mDmodList->Clear();
	mFolderTitlePairs = DFS->getFolderTitlePairs();

    if ( mFolderTitlePairs->empty() == false )
    {
        mNoDmods = false;
	    for ( std::vector< std::pair< wxString, wxString > >::const_iterator lIterator = mFolderTitlePairs->begin();
		    lIterator != mFolderTitlePairs->end();
		    lIterator++ )
	    {
		    wxStringClientData *lClientData = new wxStringClientData( lIterator->first );
		    mDmodList->Append( lIterator->second, lClientData );
	    }
    }
    else
    {
        mNoDmods = true;
    }
	return;
}

void DFArcFrame::selectDmod( void )
{
	int lSelectedDmodIndex( 0 );
    mPlayButton->Enable();
    mEditButton->Enable();
    mPackageButton->Enable();
	// Update the selection
	if ( mSelectedDmod.IsEmpty() == false )
	{
		for ( int i = 0; i < mDmodList->GetCount(); i++)
		{
			wxStringClientData* lClientData = reinterpret_cast<wxStringClientData*>( mDmodList->GetClientObject( i ) );
			wxString lCurrentSelectedDmodDirectory( lClientData->GetData() );
			if ( lCurrentSelectedDmodDirectory == mSelectedDmod )
			{
				lSelectedDmodIndex = i;
				break;
			}
		}
	}
	else
	{
        if ( mDmodList->GetCount() > 0 )
        {
		    wxStringClientData* lClientData = reinterpret_cast<wxStringClientData*>( mDmodList->GetClientObject( 0 ) );
		    mSelectedDmod = lClientData->GetData();
        }
        else
        {
            mPlayButton->Disable();
            mEditButton->Disable();
            mPackageButton->Disable();
        }
	}
	mDmodList->SetSelection( lSelectedDmodIndex );
	updateDescription( lSelectedDmodIndex );
}

void DFArcFrame::showOptions( wxCommandEvent& aEvent )
{
    Options lOptions( mConfig, &mPlayMovieValue, &mWriteIniValue, &mListAllSubdirectories,
                      &mOverrideDinkDirectory, &mSpecifiedDinkDirectory, &mDinkExe, &mEditorExe,
                      &mShowDeveloperButtons, &mCloseDfarcOnPlay );
    if ( lOptions.ShowModal() == 1 )
    {
        mImageSizer->Show( mButtonSizer, mShowDeveloperButtons );
        mImageSizer->Layout();
        if ( ::wxDirExists( mSpecifiedDinkDirectory ) == true )
        {
            // Get the full path to the directory, in case they're silly and type in a non-full path.
            chdir( mSpecifiedDinkDirectory );
            mSpecifiedDinkDirectory = ::wxGetCwd();
        }
        else
        {
            // If there's no directory, let's silently not override
            mOverrideDinkDirectory = false;            
        }
        chdir( DFS->getDinkPath().c_str() );
        updateConfig();
        refreshDmodList();
        selectDmod();
    }
    return;
}

void DFArcFrame::uninstall( wxCommandEvent& aEvent )
{
    if ( mSelectedDmod == "dink\\" )
    {
        wxMessageBox( "You must select the uninstall option from the start menu to uninstall the main game.", "Uninstall - Error", wxOK | wxICON_INFORMATION, this );
    }
    else
    {
        int lResult = wxMessageBox( "Do you want to remove all save game files?", "Uninstall - Save Game Files", wxYES_NO | wxCANCEL | wxICON_QUESTION, this );
        if ( lResult != wxCANCEL )
        {
            bool lRemoveSaveGames( false );
            if ( lResult == wxYES )
            {
                lRemoveSaveGames = true;
            }

            RecursiveDelete lRecursiveDelete( lRemoveSaveGames );
            wxDir* lDir = new wxDir( mSelectedDmod );
            lDir->Traverse( lRecursiveDelete );
            delete lDir;

            if ( lRecursiveDelete.getError() == false )
            {
                bool lSuccess( true );
                if ( lRemoveSaveGames == true )
                {
                    if ( ::wxRmdir( mSelectedDmod ) == false )
                    {
                        ::wxLogError( "Unable to remove D-Mod directory. All other files were removed." );
                        lSuccess = false;
                    }
                }
                if ( lSuccess == true )
                {
                    wxMessageBox( "D-Mod successfully uninstalled", "Uninstall - Success", wxOK | wxICON_INFORMATION, this );
                }
            }
            mSelectedDmod = "";
            refreshDmodList();
            selectDmod();
        }
    }
    return;
}