#include <wx/wx.h>
#include <wx/progdlg.h>
#include "InstallVerifyFrame.hpp"
#include "dfilesystem.hpp"
#include "bzip.hpp"
#include "tar.hpp"

BEGIN_EVENT_TABLE(InstallVerifyFrame, wxDialog)
    EVT_CLOSE(InstallVerifyFrame::onQuit)    
    EVT_BUTTON(ID_CANCEL, InstallVerifyFrame::onQuit)
    EVT_BUTTON(ID_INSTALL_BUTTON, InstallVerifyFrame::onInstall)
END_EVENT_TABLE()

InstallVerifyFrame::InstallVerifyFrame( wxWindow* aParent, const wxString& aFilePath, DFilesystem* aDinkFileSystem ) :
    wxDialog( aParent, ID_INSTALL_VERIFY_FRAME, "DFArc 2.0 - Install D-Mod", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ),
    mExtracting( true ),
    mDmodFilePath( aFilePath ),
    mDinkFileSystem( aDinkFileSystem )
{
    prepareFrame();

    this->Show( true );
    this->MakeModal( true );

    wxProgressDialog* lPrepareProgress = new wxProgressDialog( "Preparing", "The D-Mod is preparing to be installed.", 100, this, wxPD_AUTO_HIDE | wxPD_REMAINING_TIME );

    // Extract
    Bzip lBzip( mDmodFilePath, mDinkFileSystem );
    mTarFilePath = lBzip.Extract( lPrepareProgress );
    lPrepareProgress->Destroy();

    if ( mTarFilePath.Len() == 0 )
    {
        mCancelButton->Enable( true );
        mExtracting = false;
    }
    else
    {
        // Prepare the tar file for reading
        mTar = new Tar( mTarFilePath.c_str(), mDinkFileSystem );
	    mTar->ReadHeaders();

        mSelectedDmod = mTar->getInstalledDmodDirectory();

        // Get and display the dmod description
        wxString lmDmodDescription = mTar->getmDmodDescription();
        if ( lmDmodDescription.Len() == 0 )
        {
            lmDmodDescription = "No Description Available.\n\nWill install to ";
            lmDmodDescription += mTar->getInstalledDmodDirectory();
        }
        else
        {
            int lBreakChar = lmDmodDescription.Find( '\r' );
            if ( lBreakChar <= 0 )
            {
                lBreakChar = lmDmodDescription.Find( '\n' );
            }
            mDmodName = lmDmodDescription.SubString( 0, lBreakChar - 1 );
            this->SetTitle( "DFArc 2.0 - Install D-Mod - " + mDmodName );
        }
        mmDmodDescription->SetValue( lmDmodDescription );

        // Enable the cancel buttons and such
        mCancelButton->Enable( true );
        mInstallButton->Enable( true );
        mExtracting = false;
    }
    
    return;
}

InstallVerifyFrame::~InstallVerifyFrame( void )
{
    return;
}

void InstallVerifyFrame::prepareFrame( void )
{
    wxPanel* lPanel = new wxPanel( this );

    mmDmodDescription = new wxTextCtrl( lPanel, -1, "Preparing...", wxDefaultPosition,
        wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY | wxTE_LEFT | wxTE_BESTWRAP);
    mCancelButton = new wxButton( lPanel, ID_CANCEL, wxT( "Cancel" ) );
    mCancelButton->Disable();
    mInstallButton = new wxButton( lPanel, ID_INSTALL_BUTTON, wxT( "Install" ) );
    mInstallButton->Disable();

    wxBoxSizer* lHorizontalSizer = new wxBoxSizer( wxHORIZONTAL );
    lHorizontalSizer->Add( mInstallButton, wxSizerFlags(0).Left() );
    lHorizontalSizer->Add( mCancelButton, wxSizerFlags(0).Right().Border( wxLEFT, 5 ) );

	wxBoxSizer* lVerticalSizer = new wxBoxSizer( wxVERTICAL );
    lVerticalSizer->Add( mmDmodDescription, wxSizerFlags(3).Expand().Border( wxTOP | wxLEFT | wxRIGHT, 5 ) );
    lVerticalSizer->Add( lHorizontalSizer, wxSizerFlags(0).Border( wxALL, 5 ) );

	lPanel->SetSizer( lVerticalSizer );
	lPanel->SetSize( 400, 150 );
	this->SetSize( 400, 150 );
    return;
}

void InstallVerifyFrame::onQuit( wxCloseEvent &Event )
{
    if ( mExtracting == false )
    {
        remove( mTarFilePath );
        this->MakeModal( false );
        this->EndModal( 0 );
        this->Destroy();
    }
    return;
}

void InstallVerifyFrame::onQuit( wxCommandEvent &Event )
{
    if ( mExtracting == false )
    {
        remove( mTarFilePath );
        mSelectedDmod = "";
        this->MakeModal( false );
        this->EndModal( 0 );
        this->Destroy();
    }
    return;
}

void InstallVerifyFrame::onInstall( wxCommandEvent &Event )
{
    wxProgressDialog* lInstallProgress = new wxProgressDialog( "DFArc2 - Installing", "", 100, this, wxPD_AUTO_HIDE | wxPD_REMAINING_TIME );
    int lError = mTar->Extract( lInstallProgress );
    lInstallProgress->Destroy();
    if ( lError == 0 )
    {
        if ( mDmodName.Len() <= 0 )
        {
            mDmodName = "The D-Mod you selected";
        }
        wxMessageBox( mDmodName + " was successfully installed.", "Success", wxOK | wxICON_INFORMATION, this );
    }
    else
    {
        ::wxLogError( "An error occured while extracting the .dmod file." );
    }
    remove( mTarFilePath );
    this->MakeModal( false );
    this->EndModal( 0 );
    return;
}

wxString InstallVerifyFrame::getSelectedDmod( void )
{
    return mSelectedDmod;
}