#include <wx/wx.h>
#include <wx/dir.h>
#include "Options.hpp"

BEGIN_EVENT_TABLE( Options, wxDialog )
    EVT_BUTTON( ID_OKAY, Options::onOkay )
    EVT_BUTTON( ID_OP_CANCEL, Options::onCancel )
    EVT_BUTTON( ID_OVERRIDE_BROWSE, Options::onBrowse )
    EVT_CLOSE( Options::onQuit )
END_EVENT_TABLE()

Options::Options( wxConfig* aConfig, bool* aPlayMovieValue, bool* aWriteIniValue,
                  bool* aListAllSubdirectories, bool* aOverrideDinkDirectory,
                  wxString* aSpecifiedDinkDirectory, wxString* aDinkExe, wxString* aEditorExe,
                  bool* aShowDeveloperButtons, bool* aCloseDfarcOnPlay ) :
    wxDialog( 0, -1, "DFArc2 - Options", wxDefaultPosition, wxDefaultSize )
{
    mConfig = aConfig;
    mPlayMovieValue = aPlayMovieValue;
    mWriteIniValue = aWriteIniValue;
    mOverrideDinkDirectory = aOverrideDinkDirectory;
    mListAllSubdirectories = aListAllSubdirectories;
    mSpecifiedDinkDirectory = aSpecifiedDinkDirectory;
    mDinkExe = aDinkExe;
    mEditorExe = aEditorExe;
    mShowDeveloperButtons = aShowDeveloperButtons;
    mCloseDfarcOnPlay = aCloseDfarcOnPlay;

    // Get all exe files in current directory
    wxArrayString lExeFiles;
    wxDir lDir( ::wxGetCwd() );
    wxString lTempFile;
    lDir.GetFirst( &lTempFile, "*.exe", wxDIR_FILES );
    lExeFiles.Add( lTempFile );
    while ( lDir.GetNext( &lTempFile ) )
    {
        lExeFiles.Add( lTempFile );
    }

    prepareDialog( lExeFiles );
    return;
}

Options::~Options( void )
{
    return;
}

void Options::prepareDialog( wxArrayString& aExeFiles )
{
    wxPanel* lPanel = new wxPanel( this );

    // Basic options
    mWriteIniCheck = new wxCheckBox( lPanel, ID_WRITE_INI, "Dink.exe: Update dinksmallwood.ini" );
    mWriteIniCheck->SetValue( *mWriteIniValue );

    mPlayMovieCheck = new wxCheckBox( lPanel, ID_PLAY_MOVIE, "Dink.exe: Play introduction movie (requires Dink cd-rom)" );
    mPlayMovieCheck->SetValue( *mPlayMovieValue );

    mShowDeveloperCheck = new wxCheckBox( lPanel, ID_LIST_ALL, "Show developer buttons" );
    mShowDeveloperCheck->SetValue( *mShowDeveloperButtons );

    mListAllCheck = new wxCheckBox( lPanel, ID_LIST_ALL, "List all subdirectories as D-Mods" );
    mListAllCheck->SetValue( *mListAllSubdirectories );

    mCloseCheck = new wxCheckBox( lPanel, ID_LIST_ALL, "Close DFArc on play" );
    mCloseCheck->SetValue( *mCloseDfarcOnPlay );

    // Override Dink Directory
    mOverrideDinkDirectoryCheck = new wxCheckBox( lPanel, ID_OVERRIDE_CHECK, "Override the Dink Smallwood directory" );
    mOverrideDinkDirectoryCheck->SetValue( *mOverrideDinkDirectory );
    mSpecifiedDinkDirectoryBox = new wxTextCtrl( lPanel, ID_OVERRIDE_EDIT, *mSpecifiedDinkDirectory );
    mBrowseButton = new wxButton( lPanel, ID_OVERRIDE_BROWSE, "Browse" );

    wxBoxSizer* lOverrideSizer = new wxBoxSizer( wxHORIZONTAL );
    lOverrideSizer->Add( mOverrideDinkDirectoryCheck, wxSizerFlags( 0 ).Left() );
    mOverrideDinkDirectoryCheck->SetValue( *mOverrideDinkDirectory );
    lOverrideSizer->Add( mSpecifiedDinkDirectoryBox, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );
    lOverrideSizer->Add( mBrowseButton, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );

    // Dink.exe
    mDinkExeBox = new wxComboBox( lPanel, ID_DINK_EXE, *mDinkExe, wxDefaultPosition, wxDefaultSize, aExeFiles, wxCB_READONLY | wxCB_SORT );
    wxStaticText* lDinkExeLabel = new wxStaticText( lPanel, -1, "Choose Dink application" );

    wxBoxSizer* lDinkExeSizer = new wxBoxSizer( wxHORIZONTAL );
    lDinkExeSizer->Add( mDinkExeBox, wxSizerFlags( 0 ).Left() );
    lDinkExeSizer->Add( lDinkExeLabel, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );

    // Edit.exe
    mEditorExeBox = new wxComboBox( lPanel, ID_EDIT_EXE, *mEditorExe, wxDefaultPosition, wxDefaultSize, aExeFiles, wxCB_READONLY | wxCB_SORT );
    wxStaticText* lEditorExeLabel = new wxStaticText( lPanel, -1, "Choose Editor application" );

    wxBoxSizer* lEditorExeSizer = new wxBoxSizer( wxHORIZONTAL );
    lEditorExeSizer->Add( mEditorExeBox, wxSizerFlags( 0 ).Left() );
    lEditorExeSizer->Add( lEditorExeLabel, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );

    // Buttons (OK, CANCEL)
    wxButton* lCancel = new wxButton( lPanel, ID_OP_CANCEL, "Cancel" );
    wxButton* lOkay = new wxButton( lPanel, ID_OKAY, "Okay" );

    wxBoxSizer* lChoicesSizer = new wxBoxSizer( wxHORIZONTAL );
    lChoicesSizer->Add( lOkay, wxSizerFlags( 0 ).Left() );
    lChoicesSizer->Add( lCancel, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );

    // Main sizer
    wxBoxSizer* lMainSizer = new wxBoxSizer( wxVERTICAL );
    wxSizerFlags lFlags = wxSizerFlags( 0 ).Left().Border( wxTOP | wxLEFT, 10 );
	lMainSizer->Add( mWriteIniCheck, lFlags );
    lMainSizer->Add( mPlayMovieCheck, lFlags );
    lMainSizer->Add( mShowDeveloperCheck, lFlags );
    lMainSizer->Add( mListAllCheck, lFlags );
    lMainSizer->Add( mCloseCheck, lFlags );
    lMainSizer->Add( lOverrideSizer, lFlags );
    lMainSizer->Add( lDinkExeSizer, lFlags );
    lMainSizer->Add( lEditorExeSizer, lFlags );
    lMainSizer->Add( lChoicesSizer, lFlags );

	lPanel->SetSizer( lMainSizer );
	lPanel->SetSize( 450, 280 );
	this->SetSize( 450, 280 );
	return;
}

void Options::onOkay( wxCommandEvent& aEvent )
{
    *mPlayMovieValue = mPlayMovieCheck->GetValue();
    *mWriteIniValue = mWriteIniCheck->GetValue();
    *mOverrideDinkDirectory = mOverrideDinkDirectoryCheck->GetValue();
    *mListAllSubdirectories = mListAllCheck->GetValue();
    *mSpecifiedDinkDirectory = mSpecifiedDinkDirectoryBox->GetValue();
    *mShowDeveloperButtons = mShowDeveloperCheck->GetValue();
    *mDinkExe = mDinkExeBox->GetValue();
    *mEditorExe = mEditorExeBox->GetValue();
    *mCloseDfarcOnPlay = mCloseCheck->GetValue();
    EndModal( 1 );
    return;
}

void Options::onCancel( wxCommandEvent& aEvent )
{
    EndModal( 0 );
    return;
}

void Options::onQuit( wxCloseEvent& aEvent )
{
    EndModal( 0 );
    return;
}

void Options::onBrowse( wxCommandEvent& aEvent )
{
    wxDirDialog* lDirDialog = new wxDirDialog( this );
    lDirDialog->ShowModal();
    wxString lDir = lDirDialog->GetPath();
    if ( lDir.Len() > 0 )
    {
        mSpecifiedDinkDirectoryBox->SetValue( lDir );
    }
    return;
}