#include <wx/wx.h>
#include <wx/utils.h>
#include "Package.hpp"
#include "Bzip.hpp"
#include "Tar.hpp"
#include "dfilesystem.hpp"

BEGIN_EVENT_TABLE( Package, wxDialog )
    EVT_BUTTON( Package::ID_PACKAGE, Package::onOkay )
    EVT_BUTTON( Package::ID_CANCEL, Package::onCancel )
    EVT_BUTTON( Package::ID_BROWSE, Package::onBrowse )
    EVT_CLOSE( Package::onQuit )
END_EVENT_TABLE()

Package::Package( const wxString& aSelectedDmod, const wxString& aDmodDescription, DFilesystem* aDFS ) :
    wxDialog( 0, -1, "DFArc2 - Package", wxDefaultPosition, wxDefaultSize ),
    mSelectedDmod( aSelectedDmod )
{
    mDFS = aDFS;
    prepareDialog( aDmodDescription );
    return;
}

Package::~Package( void )
{
    return;
}

void Package::prepareDialog( const wxString& aDmodDescription )
{
    // Parse dmod description
    wxString lTemp( aDmodDescription );
    wxString lTitle = lTemp.BeforeFirst( '\n' );
    lTemp = lTemp.Mid( lTitle.Len() + 1 );
    wxString lInfo1 = lTemp.BeforeFirst( '\n' );
    lTemp = lTemp.Mid( lInfo1.Len() + 1 );
    wxString lInfo2 = lTemp.BeforeFirst( '\n' );
    wxString lDescription = lTemp.Mid( lInfo2.Len() + 1 );

    // GUI fun
    wxPanel* lPanel = new wxPanel( this );
    
    wxSizerFlags lLeftFlags = wxSizerFlags( 1 ).Left().Expand().Border( wxALL, 2 );
    wxSizerFlags lRightFlags = wxSizerFlags( 1 ).Right().Expand().Border( wxALL, 2 );

    // Intro
    wxStaticText* lIntroText = new wxStaticText( lPanel, -1, "Note: Unless you're making a D-Mod, you shouldn't be around here. This is for creating a .dmod file for a D-Mod you made.\n\nWill package '" + lTitle + "' located at\n" + mDFS->getDinkPath() + mSelectedDmod + '\\' );
    lIntroText->SetSizeHints( -1, 75 );
    
    // Identifier
    wxStaticText* lIdentifierText = new wxStaticText( lPanel, -1, "D-Mod filename (8 letters/numbers)" );
    mIdentifier = new wxTextCtrl( lPanel, -1, mSelectedDmod );
    mIdentifier->SetValidator( wxTextValidator( wxFILTER_ALPHANUMERIC ) );
    mIdentifier->SetMaxLength( 8 );
    wxStaticBoxSizer* lIdentifierBox = new wxStaticBoxSizer( wxHORIZONTAL, lPanel, "Identifier" );
    lIdentifierBox->Add( lIdentifierText, lLeftFlags );
    lIdentifierBox->Add( mIdentifier, lRightFlags );
    
    // Dmod.diz
    /*wxStaticText* lDmodDizTitleText = new wxStaticText( lPanel, -1, "Dmod.diz title" );
    mDmodDizTitle = new wxTextCtrl( lPanel, -1, lTitle );
    wxBoxSizer* lDmodDizTitleBox = new wxBoxSizer( wxHORIZONTAL );
    lDmodDizTitleBox->Add( lDmodDizTitleText, lLeftFlags );
    lDmodDizTitleBox->Add( mDmodDizTitle, lRightFlags );

    wxStaticText* lDmodDizInfo1Text = new wxStaticText( lPanel, -1, "Dmod.diz author/copyright information 1" );
    mDmodDizInfo1 = new wxTextCtrl( lPanel, -1, lInfo1 );
    wxBoxSizer* lDmodDizInfo1Box = new wxBoxSizer( wxHORIZONTAL );
    lDmodDizInfo1Box->Add( lDmodDizInfo1Text, lLeftFlags );
    lDmodDizInfo1Box->Add( mDmodDizInfo1, lRightFlags );

    wxStaticText* lDmodDizInfo2Text = new wxStaticText( lPanel, -1, "Dmod.diz author/copyright information 2" );
    mDmodDizInfo2 = new wxTextCtrl( lPanel, -1, lInfo2 );
    wxBoxSizer* lDmodDizInfo2Box = new wxBoxSizer( wxHORIZONTAL );
    lDmodDizInfo2Box->Add( lDmodDizInfo2Text, lLeftFlags );
    lDmodDizInfo2Box->Add( mDmodDizInfo2, lRightFlags );

    wxStaticText* lDmodDizDescriptionText = new wxStaticText( lPanel, -1, "Dmod.diz description" );
    mDmodDizDescription = new wxTextCtrl( lPanel, -1, lDescription, wxDefaultPosition,
        wxSize( -1, 100 ), wxTE_MULTILINE | wxTE_LEFT | wxTE_BESTWRAP );
    wxBoxSizer* lDmodDizDescriptionBox = new wxBoxSizer( wxHORIZONTAL );
    lDmodDizDescriptionBox->Add( lDmodDizDescriptionText, lLeftFlags );
    lDmodDizDescriptionBox->Add( mDmodDizDescription, lRightFlags );

    wxStaticBoxSizer* lDmodDizBox = new wxStaticBoxSizer( wxVERTICAL, lPanel, "Dmod.diz" );
    lDmodDizBox->Add( lDmodDizTitleBox, wxSizerFlags( 0 ).Left().Expand() );
    lDmodDizBox->Add( lDmodDizInfo1Box, wxSizerFlags( 0 ).Left().Expand() );
    lDmodDizBox->Add( lDmodDizInfo2Box, wxSizerFlags( 0 ).Left().Expand() );
    lDmodDizBox->Add( lDmodDizDescriptionBox, wxSizerFlags( 0 ).Left().Expand() );*/

    // Cleanup
    /*mCleanupSaveDat = new wxCheckBox( lPanel, -1, "Include save game files?" );
    mCleanupThumbsDb = new wxCheckBox( lPanel, -1, "Include thumbs.db files?" );
    wxStaticBoxSizer* lCleanupBox = new wxStaticBoxSizer( wxHORIZONTAL, lPanel, "Cleanup" );
    lCleanupBox->Add( mCleanupSaveDat, lLeftFlags );
    lCleanupBox->Add( mCleanupThumbsDb, lLeftFlags );*/

    // Patch
    /*mPatch = new wxCheckBox( lPanel, -1, "Create patch?" );

    wxStaticText* lPatchOriginalFileText = new wxStaticText( lPanel, -1, "Original .dmod file" );
    mPatchOriginalFile = new wxTextCtrl( lPanel, -1 );
    mPatchBrowse = new wxButton( lPanel, Package::ID_BROWSE, "Browse" );
    wxBoxSizer* lPatchOriginalBox = new wxBoxSizer( wxHORIZONTAL );
    lPatchOriginalBox->Add( lPatchOriginalFileText, wxSizerFlags( 25 ).Left().Expand().Border( wxALL, 2 ) );
    lPatchOriginalBox->Add( mPatchOriginalFile, wxSizerFlags( 55 ).Left().Expand().Border( wxALL, 2 ) );
    lPatchOriginalBox->Add( mPatchBrowse, wxSizerFlags( 20 ).Right().Expand().Border( wxALL, 2 ) );

    wxStaticBoxSizer* lPatchBox = new wxStaticBoxSizer( wxVERTICAL, lPanel, "Patch" );
    lPatchBox->Add( mPatch, lLeftFlags );
    lPatchBox->Add( lPatchOriginalBox, wxSizerFlags( 0 ).Left().Expand() );*/

    // Actions
    mCancel = new wxButton( lPanel, Package::ID_CANCEL, "Cancel" );
    mOkay = new wxButton( lPanel, Package::ID_PACKAGE, "Package" );

    wxBoxSizer* lActionBox = new wxBoxSizer( wxHORIZONTAL );
    lActionBox->Add( mOkay, wxSizerFlags( 0 ).Left() );
    lActionBox->Add( mCancel, wxSizerFlags( 0 ).Left().Border( wxLEFT, 10 ) );

    // Main sizer
    wxBoxSizer* lMainSizer = new wxBoxSizer( wxVERTICAL );
    wxSizerFlags lFlags = wxSizerFlags( 0 ).Left().Border( wxTOP | wxLEFT | wxRIGHT, 10 ).Expand();
    lMainSizer->Add( lIntroText, lFlags );
	lMainSizer->Add( lIdentifierBox, lFlags );
    // lMainSizer->Add( lDmodDizBox, lFlags );
    //lMainSizer->Add( lCleanupBox, lFlags );
    // lMainSizer->Add( lPatchBox, lFlags );
    lMainSizer->Add( lActionBox, lFlags );

	lPanel->SetSizer( lMainSizer );
	lPanel->SetSize( 450, 225 );
	this->SetSize( 450, 225 );
    return;
}

void Package::onQuit( wxCloseEvent& aEvent )
{
    EndModal( 0 );
    return;
}

void Package::onCancel( wxCommandEvent& aEvent )
{
    EndModal( 0 );
    return;
}

void Package::onBrowse( wxCommandEvent& aEvent )
{
    wxFileDialog lFileDialog( 0, "Select a .dmod file", "", "", "D-Mod files (*.dmod)|*.dmod",
        wxOPEN | wxFILE_MUST_EXIST | wxHIDE_READONLY );
     
    if ( lFileDialog.ShowModal() == wxID_OK )
    {
        mPatchOriginalFile->SetValue( lFileDialog.GetPath() );
    }
    lFileDialog.Destroy();
    return;
}

void Package::onOkay( wxCommandEvent& aEvent )
{
    bool lSuccess( true );

    if ( mIdentifier->GetValue().Len() == 0 )
    {
        ::wxLogError( "You must provide an identifier filename." );
        lSuccess = false;
    }
    /*if ( mDmodDizTitle->GetValue().Len() == 0 )
    {
        ::wxLogError( "You must provide a Dmod.diz title." );
        lSuccess = false;
    }*/
    /*if ( mPatch->GetValue() == true && ::wxFileExists( mPatchOriginalFile->GetValue() ) == false )
    {
        ::wxLogError( "You must specifiy a valid original .dmod filename." );
        lSuccess = false;
    }*/

    if ( lSuccess == true )
    {
        wxProgressDialog* lPackageProcess = new wxProgressDialog( "Packaging", "The D-Mod is being packaged.", 100, this, wxPD_AUTO_HIDE | wxPD_REMAINING_TIME );

        // Do patch stuff
        /*if ( mPatch->GetValue() == true )
        {

        }*/

        Tar lTar( 0, mDFS, mDFS->getDinkPath() + mSelectedDmod + '\\' );
        wxString lTarFilePath = lTar.Create( lPackageProcess, mIdentifier->GetValue(), true, true );
        
        // Bzip stuff
        Bzip lBzip( lTarFilePath, mDFS );
        wxString lBzipFilePath = lBzip.Create( lPackageProcess );  

        // Rename the old D-Mod
        wxString lDmodFilePath( mDFS->getDinkPath() + mIdentifier->GetValue() + ".dmod" );
        wxString lOldDmodFilePath( lDmodFilePath );
        int lCount( 1 );
        while ( ::wxFileExists( lOldDmodFilePath ) == true )
        {
            lOldDmodFilePath = wxString::Format( "%s%d.dmod", mIdentifier->GetValue(), lCount );
            lCount++;
        }
        if ( lOldDmodFilePath != lDmodFilePath )
        {
            rename( lDmodFilePath.c_str(), lOldDmodFilePath.c_str() );
        }

        // Rename the new .dmod file
        rename( lBzipFilePath.c_str(), lDmodFilePath.c_str() );

        remove( lTarFilePath.c_str() );

        // Success dialog
        wxMessageBox( mIdentifier->GetValue() + " was successfully packaged.", "Success", wxOK | wxICON_INFORMATION, this );

        EndModal( 1 );
    }
    return;
}