#include "dfarc.hpp"

//
// wxString DFile::GetTempName()
//
// Returns a temporary file name.  This has two different implementations: one
// for Windows and one for Unix.  Windows uses GetTempPath(MAX_PATH, ...) and
// Unix uses mkstemp("/tmp/dfarc.XXXXXX").
//
// Returns: The temporary file.
//
wxString DFile::GetTempName()
{
    #ifdef WIN32
        char *szTempPath = new char[MAX_PATH];
        char *szTempFilename = 0;
        wxString strTempFile;
        
        if(GetTempPath(MAX_PATH, szTempPath) == 0)
        {
            ::wxLogError("Error: GetTempPath(MAX_PATH, ...) failed in DFile::GetTempName().");
            
            delete[] szTempPath;
            return strTempFile;
        }
        
        // We got the path.  Now get the filename (haha)
        szTempFilename = new char[MAX_PATH];

        if(GetTempFileName(szTempPath, "DFArc", 0, szTempFilename) == 0)
        {
            ::wxLogError("Error: GetTempFileName(MAX_PATH, ...) failed in DFile::GetTempName().");
            
            delete[] szTempPath;
            delete[] szTempFilename;
            return strTempFile;
        }  
        
        strTempFile = szTempFilename;
        delete[] szTempPath;
        delete[] szTempFilename;
        return strTempFile;
    #else
        // UNIX.
        // mktemp is dangerous.  Use mkstemp.  Unfortunately, it opens the file,
        //     so we'll fix that.
        
        char szTempPath[18] = "";
        wxString strTempFile;
        int ifpTmp;
        
        strncpy(szTempPath, "/tmp/dfarc.XXXXXX", sizeof(szTempPath));
        
        
        if((ifpTmp = mkstemp(szTempPath)) == -1)
        {
            ::wxLogFatalError("Error: mkstemp(...) failed in DFile::AssignTempName().");
            close(ifpTmp);
            throw;
        }
        
        close(ifpTmp);
        
        strTempFile = szTempPath;
        return strTempFile;
    #endif
}

