#include "dfarc.hpp"

DFilesystem::DFilesystem( const bool* aListAllSubdirectories, const bool* aOverrideDinkDirectory, const wxString *aSpecifiedDinkDirectory ) :
    mDinkPath( findDinkDir() )
{
    mListAllSubdirectories = aListAllSubdirectories;
    mOverrideDinkDirectory = aOverrideDinkDirectory;
    mSpecifiedDinkDirectory = aSpecifiedDinkDirectory;
    mFolderTitlePairs = new std::vector< std::pair< wxString, wxString > >;
	return;
}

DFilesystem::DFilesystem( void ) :
    mDinkPath( findDinkDir() )
{
    mListAllSubdirectories = NULL;
    mOverrideDinkDirectory = NULL;
    mSpecifiedDinkDirectory = NULL;
    mFolderTitlePairs = new std::vector< std::pair< wxString, wxString > >;
	return;
}

DFilesystem::~DFilesystem( void )
{
    delete mFolderTitlePairs;
	return;
}

void DFilesystem::GetAllFiles(wxArrayString &wxasFileList, wxString strPath)
{
	wxDir *Dir = new wxDir;
	Dir->GetAllFiles(strPath, &wxasFileList, "", wxDIR_DEFAULT);
	delete Dir;
}


bool DFilesystem::IsValidFilename(const char *sz)
{
	char *c = const_cast<char*>(sz);
	for(unsigned int i = 0; i < strlen(sz); ++i, ++c)
	{
		if(*c == '/' || *c == '\\' || *c == ':' || *c == '*' || *c == '?' ||
			*c == '<' || *c == '>' || *c == '|')
		{
			return false;
		}
	}
	return true;
}



wxString DFilesystem::findDinkDir( void )
{    
	// First see if it's in the registry.
#ifdef WIN32    
	unsigned long int iszPathSize = 512;
	unsigned long int iRegType = REG_SZ;
	unsigned long int liError;
	unsigned char *szPath = new unsigned char[512];
	HKEY hDS;

	memset(szPath, 0, 512);

	// Open the registry path.
	liError = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\RTSOFT\\Dink Smallwood",
		0, KEY_READ, &hDS);

	while(1)
	{
		// Grab the path.
		if(RegQueryValueEx(hDS, "Path", 0, &iRegType, szPath,
			&iszPathSize) != ERROR_MORE_DATA)
		{
			break;
		}
		else
		{
			// If the path is too small, make the buffer bigger.
			iszPathSize += 512;
			delete[] szPath;
			szPath = new unsigned char[iszPathSize];
			memset(szPath, 0, iszPathSize);
		}
	}

	// Close the registry.
	RegCloseKey(hDS);

	if(liError == ERROR_SUCCESS && szPath[0] != '\0')
	{        
		wxString str = szPath;
		delete[] szPath;

		DFilesystem::FixSlashes(str);
		return str;
	}
#endif

	::wxLogWarning("Warning: you do not have a Dink Smallwood Path registry key.  Trying to find"
		" the deprecated dinksmallwood.ini file (may be inaccurate).");


	// We don't have the registry key.  See if we can use the deprecated dinksmallwood.ini file.
	std::fstream f_In;
	wxString strCurDir;
	wxString strSysRoot;
	wxString strBuf;
	char ch;

#ifdef WIN32
	// Get %SystemRoot% and set our magic filename.
	if(!getenv("SystemRoot"))
	{
		// Check for Win9x's %windir%
		if(!getenv("windir"))
		{
			// Abort.
			strSysRoot = "C:/Windows";

			if(!::wxDirExists(strSysRoot))
			{
				::wxLogError("Error: Could not find your system directory.  You do not have"
					" the environment variables %%SystemRoot%% or %%windir%%!");
				throw;
			}
		}
		else
		{
			strSysRoot = getenv("windir");
		}
	}
	else
	{
		strSysRoot = getenv("SystemRoot");
	}

	DFilesystem::FixSlashes(strSysRoot);

#else

	// Linux.  We'll make sure it exists.
	wxFileName *wxfn = new wxFileName();
	strSysRoot = wxfn->GetHomeDir();
	DFilesystem::FixSlashes(strSysRoot);

	strSysRoot += ".dink\\";

	delete wxfn;

	if(!::wxDirExists(strSysRoot))
	{
		::wxLogFatalError("Error: Dink directory not found.  Looking in ~/.dink/.");
		throw;
	}

	// Found it.  Return.
	return strSysRoot;

#endif


	strSysRoot += "dinksmallwood.ini";


	// Open dinksmallwood.ini
	if(!::wxFileExists(strSysRoot))
	{
		::wxLogFatalError("Error: Dinksmallwood.ini not found.  Please run the main game and try running this program again.");
		throw;
	}

	f_In.open(strSysRoot.c_str(), std::ios::in);

	if(!f_In.good())
	{
		::wxLogFatalError("Error: Bad file handle for %s in DFilesystem::ReturnDinkDir() method!", strSysRoot.c_str());
		throw;
	}

	// Parse
	while(!f_In.eof())
	{
		while(1)
		{
			ch = f_In.get();

			if(ch == '\n')
			{
				break;
			}

			if(ch != '\r' && ch != '\t')
			{
				strBuf.Append(ch, 1);
			}
		}

		// Parse String.
		if(strBuf.GetChar(0) == '[')
		{
			// We can't use this line.  Start over.
			strBuf.Clear();
			continue;
		}

		// Search for any redeemable qualities.  We'll look for a /.
		for(unsigned int i = 0; i < strBuf.Length(); ++i)
		{
			if(strBuf.GetChar(i) == '/' || strBuf.GetChar(i) == '\\' || strBuf.GetChar(i) == ':')
			{
				f_In.close();

				DFilesystem::FixSlashes(strBuf);
				return strBuf;
			}
		}

		// We can skip any other parts of the file.
	}


	// We shouldn't get here, but we'll try and save the program.
	f_In.close();

	DFilesystem::FixSlashes(strBuf);
	return strBuf;
}


//
// void DFilesystem::PopulateDirectoryArray(wxString &Path) [private]
//
// This finds all of the directories in the specified directory.  wxasDirList 
// is populated with the results.
//
// returns: Nothing.
//
void DFilesystem::populateFolderTitlePairs( wxString& aDinkPath )
{
	mFolderTitlePairs->clear();
	// In case a path across the hard drive is specified, save the current dir.
	wxString lPreviousWorkingDirectory = ::wxGetCwd();
	::wxSetWorkingDirectory( aDinkPath );
	wxDir lDirectory( aDinkPath );

	if ( lDirectory.IsOpened() == false )
	{
		::wxLogError( "Error: Could not open directory %s for traverse.", aDinkPath.c_str() );
	}

	wxString lCurrentFolder;
	int iNumResults = 0;

	// Recurse through and grab dir names.
	bool lDirectoryExists = lDirectory.GetFirst( &lCurrentFolder );

	while ( lDirectoryExists == true )
	{
		if ( ::wxDirExists( lCurrentFolder ) )
		{
			addFolderTitlePair( lCurrentFolder );
		}
		lDirectoryExists = lDirectory.GetNext( &lCurrentFolder );
	}

	::wxSetWorkingDirectory( lPreviousWorkingDirectory );
	return;
}

void DFilesystem::addFolderTitlePair( wxString aFolder )
{
	// Append a trailing new line if one doesn't already exist
	if ( aFolder.Last() != '/' || aFolder.Last() != '\\' )
	{
		aFolder += '\\';
	}

	wxString lTitle( aFolder );

	wxString lMapDatPath( aFolder + "map.dat" );
	wxString lDinkDatPath( aFolder + "dink.dat" );
	wxString lDmodDizPath( aFolder + "dmod.diz" );
	wxString lStoryPath( aFolder + "story" );

	// Determine if this is a likely D-Mod by searching for common D-Mod files
	if ( /*::wxFileExists( lMapDatPath ) ||
		 ::wxFileExists( lDinkDatPath ) ||*/
		 ::wxFileExists( lDmodDizPath ) /*||
 		 ::wxDirExists( lStoryPath )*/ ||
         *mListAllSubdirectories )
	{
		// Save the diz file in the DIZ path array.
		//wxasDIZPathList.Add(dmodDiz);

		// If the file has a dmod.diz, let's replace the title
		if( ::wxFileExists( lDmodDizPath ) )
		{
            lTitle = getDmodDescription( aFolder );
		}
		std::pair< wxString, wxString > lPair( aFolder, lTitle );
		mFolderTitlePairs->push_back( lPair );
	}
	return;
}

const std::vector< std::pair< wxString, wxString> >* DFilesystem::getFolderTitlePairs( void )
{
	populateFolderTitlePairs( getDinkPath() );
	return mFolderTitlePairs;
}

wxString DFilesystem::getDinkPath( void )
{
    wxString lDinkPath( mDinkPath );
    if ( *mOverrideDinkDirectory == true )
    {
        lDinkPath = *mSpecifiedDinkDirectory;
    }
    FixSlashes( lDinkPath );
	return lDinkPath;
}

wxString DFilesystem::getDmodDescription( wxString aDmodFolder )
{
    wxString lDescription;
    wxString lDmodDizPath( aDmodFolder + "dmod.diz" );
    if( ::wxFileExists( lDmodDizPath ) )
	{
        std::fstream lFileStream;
		lFileStream.open( lDmodDizPath.c_str(), std::ios::in );
		while( 1 )
		{
			char ch = lFileStream.get();
			if ( ch == '\n' || ch == '\r' || ch == EOF )
			{
				break;
			}
			lDescription += ch;
		}
		lFileStream.close();
		lFileStream.clear();
	}
    return lDescription;
}