#if !defined ( TAR_HPP )
#define TAR_HPP
#include <wx/progdlg.h>
#include <vector>
#include "dfile.hpp"
#include "dfilesystem.hpp"

class TarHeader
{
public:

    char Name[100];      // 100
    char Mode[8];        // 108
    char Uid[8];         // 116
    char Gid[8];         // 124
    char Size[12];       // 136
    char Mtime[12];      // 148
    char Chksum[8];      // 156
    char Linkflag;       // 157
    char Linkname[100];  // 257
    char Magic[8];       // 265
    char Uname[32];      // 297
    char Gname[32];      // 329
    char Devmajor[8];    // 337
    char Devminor[8];    // 345
    char Padding[167];   // 512
};


class TarRecord
{
public:

    char Name[100];
    int iFilePosBegin;
    int iFileSize;
};



class Tar : public DFile
{
public:

    Tar(const char *szFile, DFilesystem *DFSPtr, const char *szCompressDir) : 
        DFile(szFile, DFSPtr)
    {
        strCompressDir = szCompressDir;
        bCanCompress = true;
        this->DFSPtr = DFSPtr;
    }
       
    Tar(const char *szFile, DFilesystem *DFSPtr) : DFile(szFile, DFSPtr)
    {
        bCanCompress = false;
        this->DFSPtr = DFSPtr;
    }

    ~Tar()
    {
    }

    wxString Create( wxProgressDialog* aProgressDialog, const wxString& aFolderName, const bool& aIncludeSaveDat, const bool& aIncludeThumbsDb );
    int Extract( wxProgressDialog* aProgressDialog );
    // tarmisc.cpp:
    int ReadHeaders( void );
	wxArrayString mPathList;
    wxString getInstalledDmodDirectory( void );
    wxString getmDmodDescription( void );
private:
    
    void FillHeader(wxString &strFile, const wxString& lDestinationFile, TarHeader &Record);
    void DecToOct(int n, char *sz, int iMaxSize);
    const char* OctToDec(int n);
    const char* ToString(int n);
    int RoundTo512(int n);
    bool PadWithZeros(char *s, int iszSize, int iAcheiveSize);
    bool VerifyChecksum(TarHeader *Header);
    int ExtractData( std::fstream &f_In, wxProgressDialog* aProgressDialog );

    // True if the proper constructor was used.  The compression method
    // will not screw up if this flag is set to true.
    bool bCanCompress;
    
    // The directory to compress.  
    wxString strCompressDir;
	std::vector<TarRecord> TarRecords;
    DFilesystem *DFSPtr;
    wxString mInstalledDmodDirectory;
    wxString mmDmodDescription;
    
};


#endif